@@ -21,7 +21,7 @@ import structlog
 import yaml
 from requests.exceptions import HTTPError
 
-from db_contrib_tool.plugin import PluginInterface, Subcommand
+from db_contrib_tool.plugin import PluginInterface, Subcommand, SubcommandResult
 from db_contrib_tool.setup_repro_env import config, download, github_conn
 from db_contrib_tool.utils import evergreen_conn, is_windows
 
@@ -30,6 +30,15 @@ SUBCOMMAND = "setup-repro-env"
 LOGGER = structlog.getLogger(__name__)
 
 
+class SetupReproEnvError(Exception):
+    """Errors in setup_repro_env.py.
+
+    The base class of exceptions for this file/subcommand.
+    """
+
+    pass
+
+
 def setup_logging(debug=False):
     """Enable logging."""
     log_level = logging.DEBUG if debug else logging.INFO
@@ -92,7 +101,6 @@ class EvgURLInfo(NamedTuple):
 class SetupReproEnv(Subcommand):
     """Main class for the setup repro environment subcommand."""
 
-    # pylint: disable=too-many-instance-attributes
     def __init__(
         self,
         download_options,
@@ -172,7 +180,7 @@ class SetupReproEnv(Subcommand):
                 "If you're running it from the root of the mongo repo and still seeing"
                 " this error, please reach out in #server-testing slack channel."
             )
-            exit(1)
+            raise SetupReproEnvError()
         sys.path.append(os.path.join(os.getcwd(), "buildscripts", "resmokelib"))
         try:
             import multiversionconstants as _multiversionconstants
@@ -211,7 +219,7 @@ class SetupReproEnv(Subcommand):
 
         for version in self.versions:
             LOGGER.info("Setting up version.", version=version)
-            LOGGER.info("Fetching download URL from Evergreen.")
+            LOGGER.info("Fetching download URLs from Evergreen.")
 
             try:
                 self.platform = (
@@ -224,43 +232,42 @@ class SetupReproEnv(Subcommand):
                     urls_info = self.get_latest_urls(version)
                 if self.use_latest and not urls_info.urls:
                     LOGGER.warning(
-                        "Latest URL is not available, falling back"
-                        " to getting the URL from 'mongodb-mongo-master'"
+                        "Latest URLs are not available, falling back"
+                        " to getting the URLs from 'mongodb-mongo-master'"
                         " project preceding the merge-base commit."
                     )
                     merge_base_revision = get_merge_base_commit(version)
                     urls_info = self.get_latest_urls("master", merge_base_revision)
                 if not urls_info.urls:
                     LOGGER.warning(
-                        "Latest URL is not available or not requested,"
-                        " falling back to getting the URL for a specific"
+                        "Latest URLs are not available or not requested,"
+                        " falling back to getting the URLs for a specific"
                         " version."
                     )
                     urls_info = self.get_urls(version, self.variant)
                 if not urls_info.urls:
-                    LOGGER.error("URL is not available for the version.", version=version)
-                    exit(1)
+                    raise SetupReproEnvError("URLs are not available for the version.")
 
                 urls = urls_info.urls
 
-                downloaded_versions.append(urls_info.evg_version_id)
-
                 bin_suffix = self._get_bin_suffix(version, urls["project_identifier"])
                 # Give each version a unique install dir
                 install_dir = os.path.join(self.install_dir, version)
 
-                self.download_and_extract_from_urls(urls, bin_suffix, install_dir)
+                self.download_and_extract(urls, bin_suffix, install_dir)
+                downloaded_versions.append(urls_info.evg_version_id)
             except (
                 github_conn.GithubConnError,
                 evergreen_conn.EvergreenConnError,
                 download.DownloadError,
+                SetupReproEnvError,
             ) as ex:
                 LOGGER.error(ex)
-                exit(1)
-
+                LOGGER.error("Setup version failed.", version=version)
             else:
                 LOGGER.info("Setup version completed.", version=version)
-                LOGGER.info("-" * 50)
+
+            LOGGER.info("-" * 50)
 
         if self._is_windows:
             self._write_windows_install_paths(self._windows_bin_install_dirs)
@@ -268,7 +275,15 @@ class SetupReproEnv(Subcommand):
         if self.evg_versions_file:
             self._write_evg_versions_file(self.evg_versions_file, downloaded_versions)
 
-    def download_and_extract_from_urls(self, urls, bin_suffix, install_dir):
+        if len(downloaded_versions) < len(self.versions):
+            LOGGER.error(
+                "Some versions were not able to setup.",
+                failed_versions=[v for v in self.versions if v not in downloaded_versions],
+            )
+            return SubcommandResult.FAIL
+        return SubcommandResult.SUCCESS
+
+    def download_and_extract(self, urls, bin_suffix, install_dir):
         """Download and extract values indicated in `urls`."""
         artifacts_url = urls.get("Artifacts", "") if self.download_artifacts else None
         binaries_url = urls.get("Binaries", "") if self.download_binaries else None
@@ -278,14 +293,16 @@ class SetupReproEnv(Subcommand):
             if self.download_python_venv
             else None
         )
-        download_symbols_url = None
+        symbols_url = (
+            urls.get(" mongo-debugsymbols.tgz", "")
+            or urls.get("mongo-debugsymbols.tgz", "")
+            or urls.get(" mongo-debugsymbols.zip", "")
+            or urls.get("mongo-debugsymbols.zip", "")
+            if self.download_symbols
+            else None
+        )
 
-        if self.download_symbols:
-            download_symbols_url = urls.get(" mongo-debugsymbols.tgz", None)
-            if not download_symbols_url:
-                download_symbols_url = urls.get(" mongo-debugsymbols.zip", None)
-
-        if self.download_symbols and not download_symbols_url:
+        if self.download_symbols and not symbols_url:
             raise download.DownloadError("Symbols download requested but not URL available")
 
         if self.download_artifacts and not artifacts_url:
@@ -302,7 +319,7 @@ class SetupReproEnv(Subcommand):
         self.setup_mongodb(
             artifacts_url,
             binaries_url,
-            download_symbols_url,
+            symbols_url,
             python_venv_url,
             install_dir,
             bin_suffix,
@@ -382,10 +399,10 @@ class SetupReproEnv(Subcommand):
 
     def get_urls(self, version: str, buildvariant_name: Optional[str] = None) -> EvgURLInfo:
         """Return multiversion urls for a given version (as binary version or commit hash or evergreen_version_id)."""
-        multiversionconstants = self.get_multiversionconstants()
+        evergreen_projects = evergreen_conn.get_evergreen_projects(self.evg_api)
 
         evg_version = evergreen_conn.get_evergreen_version(
-            multiversionconstants.EVERGREEN_PROJECTS, self.evg_api, version
+            evergreen_projects, self.evg_api, version
         )
         if evg_version is None:
             git_tag, commit_hash = github_conn.get_git_tag_and_commit(
@@ -393,7 +410,7 @@ class SetupReproEnv(Subcommand):
             )
             LOGGER.info("Found git attributes.", git_tag=git_tag, commit_hash=commit_hash)
             evg_version = evergreen_conn.get_evergreen_version(
-                multiversionconstants.EVERGREEN_PROJECTS, self.evg_api, commit_hash
+                evergreen_projects, self.evg_api, commit_hash
             )
         if evg_version is None:
             return EvgURLInfo()
@@ -433,39 +450,43 @@ class SetupReproEnv(Subcommand):
         link_dir=None,
         install_dir_list=None,
     ):
-        # pylint: disable=too-many-arguments
         """Download, extract and symlink."""
 
-        for url in [artifacts_url, binaries_url, symbols_url, python_venv_url]:
-            if url is not None:
+        urls = [
+            url
+            for url in [artifacts_url, binaries_url, symbols_url, python_venv_url]
+            if url is not None
+        ]
 
-                def try_download(download_url):
-                    tarball = download.download_from_s3(download_url)
-                    download.extract_archive(tarball, install_dir)
-                    os.remove(tarball)
+        for url in urls:
 
-                try:
-                    try_download(url)
-                except Exception as err:  # pylint: disable=broad-except
-                    LOGGER.warning(
-                        "Setting up tarball failed with error, retrying once...", error=err
-                    )
-                    time.sleep(1)
-                    try_download(url)
+            def try_download(download_url):
+                tarball = download.download_from_s3(download_url)
+                download.extract_archive(tarball, install_dir)
+                os.remove(tarball)
 
-        if binaries_url is not None:
-            if not link_dir:
-                raise ValueError("link_dir must be specified if downloading binaries")
+            try:
+                try_download(url)
+            except Exception as err:
+                LOGGER.warning("Setting up tarball failed with error, retrying once...", error=err)
+                time.sleep(1)
+                try_download(url)
 
-            if not is_windows():
-                link_dir = download.symlink_version(bin_suffix, install_dir, link_dir)
-            else:
-                LOGGER.info(
-                    "Linking to install_dir on Windows; executable have to live in different working"
-                    " directories to avoid DLLs for different versions clobbering each other"
-                )
-                link_dir = download.symlink_version(bin_suffix, install_dir, None)
-            install_dir_list.append(link_dir)
+        if binaries_url is None:
+            return
+
+        if not link_dir:
+            raise ValueError("link_dir must be specified if downloading binaries")
+
+        if is_windows():
+            LOGGER.info(
+                "Linking to install_dir on Windows; executable have to live in different working"
+                " directories to avoid DLLs for different versions clobbering each other"
+            )
+            link_dir = download.symlink_version(bin_suffix, install_dir, None)
+        else:
+            link_dir = download.symlink_version(bin_suffix, install_dir, link_dir)
+        install_dir_list.append(link_dir)
 
     def get_buildvariant_name(self, major_minor_version):
         """
